################################################################################################
# script  : set environment variables for postgres connection strings used by libpq, libpqxx
#           http://www.postgresql.org/docs/current/static/libpq-envars.html
# author  : venkat@unitrends.com
# created : Thu Dec  5 06:25:48 EST 2013
# params  : none
# output  : none
# returns : 1 on error, 0 on success
# bugzilla: 18389
# notes   : 
################################################################################################

#_____________________________________________________________
# description: pgconfig_latest returns the latest postgresql
#              configuration path/file from /etc/rc.d/init.d/
#              if present, else "none"
# return type: string
#_____________________________________________________________
function pgconfig_latest() {


  ## there could be more than one postgresql version
  ## look for available ones, if none is available return "none"
  local fpattern=$1
  local flist=(`find $INITD -name "$fpattern" -type f`)
  if [ ${#flist[*]} -lt 1 ]; then
    echo "none"
    return
  fi

  local pgcur=70
  local cnt=0
  declare -a pgver

  ## loop over the list of files returned by find,
  ## find major version 9.2, multiply by 10 (92),
  ## and store results in pgver array
  ## NOTE: PGVERSION is set by postgres SPEC file and
  ##       we don't modify it - just use it
  for rfile in ${flist[@]}; do
    ver=`grep "PGVERSION=" $rfile | sed -e "s/PGVERSION=//g" -e 's/^\([0-9]*\.[0-9]*\).*$/\1/' | awk '{print $1*10}'`
    pgver[${#pgver[*]}]=$ver
    cnt=`expr $cnt + 1`
  done

  ## loop again and get the latest version
  ## by construction both pgver and flist
  ## arrays have same number of elements
  for i in `seq 0 ${#pgver[*]}`; do
    if [[ ${pgver[$i]} -gt $pgcur ]]; then
      cnt=$i
      pgcur=${pgver[$i]}
    fi
  done

  ## return the latest config file
  echo ${flist[$cnt]}
}


#_____________________________________________________________
# main program
#

E_WARN=137
E_OK=0

INITD=/etc/rc.d/init.d
. $INITD/functions

pg_result=$E_OK

bpbase=`grep BPDIR= /etc/default/bp.ini | cut -d= -f2`
pgdir=$bpbase/data

## match pattern for postgres* config
fpattern="postgresql*"
pg_prompt="config matching $fpattern"

## get the config file
pgconfig_file=`pgconfig_latest "$fpattern"`

## unitrends default username "needs to change -- todo"
PGUSER="postgres"; export PGUSER=$PGUSER;

## if ok, then set the env variables else don't throw an error. we are looking
## at a really old version of postgres (<= 8.1.23) we just set defaults and  return with error_code 1
if [  "$pgconfig_file" == "none" ]; then
  #echo -n "Can't find $pg_prompt"; echo_warning; echo
  # let's hope that the defaults are true. this block should
  # never be executed in modern appliances. 
  # and even if it does, we can be certain its true up to PG >=7.1 
  PGENGINE="/usr/bin"; export PGENGINE=$PGENGINE
  PGPORT=5432; export PGPORT=$PGPORT
  PGVERSION=`postgres -V | awk '{print $3}' | tr -d '\n'`
  PGMAJORVERSION=`echo "$PGVERSION" | sed 's/^\([0-9]*\.[0-9]*\).*$/\1/'`; export PGMAJORVERSION=$PGMAJORVERSION
  pg_result=$E_WARN
else
  ## we want to keep some variables as is and customize others untouched: those that need to remain as is
  PGENGINE=`grep "PGENGINE=" $pgconfig_file | sed -e "s/PGENGINE=//g"`; export PGENGINE=$PGENGINE
  PGVERSION=`grep "PGVERSION=" $pgconfig_file | sed -e "s/PGVERSION=//g"`; export PGVERSION=$PGVERSION
  PGMAJORVERSION=`echo "$PGVERSION" | sed 's/^\([0-9]*\.[0-9]*\).*$/\1/'`; export PGMAJORVERSION=$PGMAJORVERSION
  PGPORT=`grep "PGPORT=" $pgconfig_file | sed -n 's/^PGPORT=//p'`; export PGPORT=$PGPORT
fi

# customized: define data directory, start-up log and pg_upgrade log files

## unitrends default username "needs to change -- todo"
PGUSER="postgres"; export PGUSER=$PGUSER;

## unitrends default database name
PGDATABASE="bpdb"; export PGDATABASE=$PGDATABASE;

## http://www.postgresql.org/docs/current/static/libpq-connect.html#LIBPQ-CONNECT-HOST
PGHOST="localhost"; export PGHOST=$PGHOST;

## see bug #18483. we need this to prevent cryptoDaemon coredump on startup in CentOS6
PGSSLMODE="allow"; export PGSSLMODE=$PGSSLMODE;

## can be re-defined pg_service.conf  (see PGSERVICE env variable below)
PGCONNECT_TIMEOUT=5; export PGCONNECT_TIMEOUT=$PGCONNECT_TIMEOUT

## default connnection port for pgbouncer
PGBPORT=6432; export PGBPORT=$PGBPORT

## unitrends default data directory
PGDATA="$pgdir"; export PGDATA=$PGDATA

## startup logs for postmaster start/stops
PGLOG="$pgdir/pgstartup.log"; export PGLOG=$PGLOG

## upgrade log -- for future use
PGUPLOG="$pgdir/pgupgrade.log"; export PGUPLOG=$PGUPLOG

## in case we need to override default service configuration parameters we need PGSERVICE 
## http://www.postgresql.org/docs/current/static/libpq-pgservice.html
PGSYSCONFDIR="/etc/sysconfig/pgsql"; export PGSYSCONFDIR=$PGSYSCONFDIR;
## todo: enable PGSERVICE only after modifying setup_postgresql.sh to set upsilon
## connection paramters - otherwise we break the build
# PGSERVICE="upsilon"; export PGSERVICE=$PGSERVICE

PGSERVICEFILE="$PGSYSCONFDIR/pg_service.conf"; export PGSERVICEFILE=$PGSERVICEFILE;

## set time zone (assuming it was done in the initial setup using RRC or cmdline)
## moved to cmc_datetime (see bug 18604)
#PGTZ=`date +%Z`; export PGTZ=$PGTZ;

## return result
return $pg_result

## end of program

