# Copyright (C) 1997 by UniTrends Software Corporation
# All Rights Reserved
# Author : Ken Carter  11/97
# Modifed: SWS 11/2001 
#          Yuri 08/2004
#          Linda 05/2007 re-written to use PostgreSQL functions
# DBupdate.sh
# Controlling Variables in the master.ini file.
# [DataBase]
#   UpdateCmd=$BP_BINDIR/DBupdate.sh
#   UpdateDays=7
#   LastUpdate=mm/dd/yyyy-hh:mm
#
# This script is used by tasker to do any necessary purging or database
# work for the Backup Professional product.  Tasker uses the UpdateCmd 
# field in the master.ini file section [DataBase] to execute this script.
# Hence the UpdateCmd=$BP_BINDIR/DBupdate.sh would need to be set for tasker
# to run this.  Tasker will run the UpdateCmd every UpdateDays interval.  
#
# UpdateDays is set to 7 initially in the master.ini file.  Normally tasker
# will run this script 1 minute past midnight every week from the day it was
# installed.  One factor that effects tasker's ability to run this script is
# that there can be no jobs in the job table prior to running it.  If there 
# are jobs tasker will wait until all of the jobs have finished and then 
# start the UpdateCmd.  This means that if a schedule was to launch at 23:50
# hours that tasker would find jobs in the job table at 1 minute past midnight
# and would wait until the scheduled backups were complete and then start the
# UpdateCmd.
#
# If you want to set a different time for tasker to start the UpdateCmd edit
# the master.ini file variable LastUpdate=mm/dd/yyyy-hh:mm in the [DataBase]
# section to the desired "hh:mm" time of the day to run.
#
# The results of the script will be mailed to the Unattended Backup mail user
# with a subject of "Database Update"
# 
# Exit code of 0  is success
#
# Each call to a PostgreSQL function is a separate transaction.  If any step
# fails, no partial database changes will remain as a result of that step.
#

function elapsed_time()
{
	total_sec=`expr $1 - $2`
	hours=`expr $total_sec / 3600`
	min=`expr \( $total_sec % 3600 \) / 60`
	sec=`expr \( $total_sec % 3600 \) % 60`
	if [ $hours -ne 1 ]
	then
		hr="hours"
	else
		hr="hour"
	fi
	if [ $min -ne 1 ]
	then
		mm="minutes"
	else
		mm="minute"
	fi
	if [ $sec -ne 1 ]
	then
		ss="seconds"
	else
		ss="seconds"
	fi

	echo "$hours $hr $min $mm $sec $ss"
}

eval `grep BPDIR /etc/default/bp.ini` 
eval `${BPDIR}/bin/bputil -e -c`

PURGE=`$BP_BINDIR/bputil -g -b DataBase NetpathsPurge 0 $BP_MASTER_INI`
DAYS=`$BP_BINDIR/bputil -g DataBase PurgeDays 4 $BP_MASTER_INI`

# Using the -b option with bputil to get the value causes PURGE to be either 0 or 1
# even if it was really some value like no|NO|No|yes|YES|Yes|TRUE|True|False|FALSE
case $PURGE in
	0)
		PURGE=false
		;;
	1)
		PURGE=true
		;;
	*) ;;		
esac

overall_start_time=`date +%s`

echo "
DBupdate Start time: `date`"

echo "======================================================================"
echo "Purging empty backups older than $DAYS days..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select bp.PurgeEmptyBackups($DAYS, $PURGE)" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
echo "Purging logically deleted backups..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select bp.ReallyPurgeBackups($PURGE)" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
echo "Purging unreferenced filesets..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select bp.UnreferencedFilesets(true,$PURGE)" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
echo "Purging unreferenced details..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select bp.UnreferencedDetails(true)" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
echo "Marking and purging unreferenced netpaths..."
echo "NOTE: this is a lengthy operation and should not be disturbed!"
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -v ON_ERROR_STOP=TRUE -c "select * from bp.MarkNetpathsForPurge()" bpdb
ret=$?
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

if [ $ret -ne 0 ]
then
	echo "ERROR: marking and purging netpaths failed with error $ret."
	echo "No netpaths, file names, or directory names were purged."
	echo "Restoring Library purge status..."
	sudo -u postgres psql -c "select bp.SetLibraryPurgeStatus(0,'')" bpdb
	overall_end_time=`date +%s`
	echo 
	echo "DBupdate End time: `date`"
	echo Total elapsed time: `elapsed_time $overall_end_time $overall_start_time`
	exit $ret 
fi

echo "======================================================================"
echo "Purging unreferenced file names..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select bp.PurgeFilenames()" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
echo "Purging unreferenced directory names..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select bp.PurgeDirnames()" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
echo "Finding any backups that may need to be imported..."
echo "Start time: `date`"
start_time=`date +%s`
sudo -u postgres psql -c "select * from bp.ImportTapeCreateReport() \
		as (tape bigint, client varchar, backup bigint)" bpdb
end_time=`date +%s`
echo Elapsed time: `elapsed_time $end_time $start_time`

echo "======================================================================"
overall_end_time=`date +%s`
echo "
DBupdate End time: `date`"
echo Total elapsed time: `elapsed_time $overall_end_time $overall_start_time`
exit 0

