#!/usr/bin/php
<?php

// This script cleans up any duplicate grandclients and/or application
// instances that may exist due to a race condition when multiple
// vaultServer processes simultaneously import backups for the same client
// or application.  It then adds a unique constraint on the identity column
// of application_instances to prevent future occurrences.
//
// Returns: 0 on success, 1 on failure.

echo "======================================================================\n";
echo "Perform cleanup of grandclient duplicates on ".date('D M j Y h:i A')."\n";
$debug=false;
$dbconn_string = "port=5432 dbname=bpdb user=postgres";
$dbconn = pg_connect($dbconn_string);
if($dbconn === FALSE) {
	echo "ERROR: Failed to connect to the database.\n";
	exit(1);
}

// See if the constraint has already been applied.
$q = "SELECT * FROM pg_catalog.pg_constraint ";
$q.= "WHERE conname = 'application_instances_identity_unique'";
$res = pg_query($dbconn, $q);
if ($res === false) {
	echo "ERROR: failed to access constraint info:\n";
	print pg_last_error($dbconn)."\n";
	exit(1);
}
if (pg_num_rows($res) > 0) {
	echo "Cleanup is not required, constraint has already been added.\n";
	exit(0);
}
else {
	if($debug) echo "Uniqueness constraint on identity doesn't exist...\n";
}

// See if any duplicates exist to clean up.  The number of row returned by
// this query is the number of identities that have duplicates.
$q = "SELECT * FROM (SELECT COUNT(node_no) FROM bp.nodes ";
$q.= "JOIN bp.application_instances USING(node_no) WHERE (properties & 32)=32 ";
$q.= "GROUP BY identity) AS FOO WHERE COUNT > 1";
$res = pg_query($dbconn, $q);
if ($res === false) {
	echo "ERROR: failed to see if duplicates exist:\n";
	print pg_last_error($dbconn)."\n";
	exit(1);
}
$ndups = pg_num_rows($res);
if ($ndups == 0) {
	echo "No duplicates found, cleanup is not required.\n";
	$q = "ALTER TABLE bp.application_instances ADD CONSTRAINT ";
	$q.= "application_instances_identity_unique UNIQUE(identity)";
	$res = pg_query($dbconn, $q);
	if ($res === false) {
		echo "ERROR: failed to set uniqueness constraint on identity:\n";
		print pg_last_error($dbconn)."\n";
		exit(1);
	}
	else {
		echo "Succesfully added uniqueness constraint on identity.\n";
		exit (0);
	}
}
else {
	printf("%d duplicated identit%s found.\n", $ndups, $ndups==1? "y":"ies");
}
// First, find duplicate grandclients (file-level identity, app_id=1).
$q = "SELECT identity FROM (SELECT COUNT(node_no), identity ";
$q.= "FROM bp.nodes JOIN bp.application_instances USING(node_no) ";
$q.= "WHERE (properties & 32)=32 AND app_id=1 GROUP BY identity) AS foo ";
$q.= "WHERE COUNT > 1";
$res = pg_query($dbconn, $q);
if ($res === false) {
	echo "ERROR: failed to query for duplicate grandclients:\n";
	print pg_last_error($dbconn)."\n";
	exit(1);
}

// Cookie bypass since we'll call bp_delete_client().
bp_bypass_cookie(3);

$nrows = pg_num_rows($res);
printf("%d duplicated grandclient identit%s found.\n", $nrows, 
		$nrows == 1 ? "y" : "ies");
for ($i = 0; $i < $nrows; $i++) {
	if (pg_field_is_null($res, $i, "identity")) {
		echo "Skipping NULL identity.\n";
		continue;
	}
	// Get the list of node_nos with this file-level identity.
	$identity = pg_fetch_result($res, $i, "identity");
	$q = "SELECT node_no FROM bp.application_instances WHERE identity = '";
	$q.= $identity . "'";
	$r2 = pg_query($dbconn, $q);
	if ($r2 === false) {
		echo "ERROR: failed to query for node_no of duplicate grandclients:\n";
		print pg_last_error($dbconn)."\n";
		bp_destroy_cookie();
		exit(1);
	}
	// This shouldn't happen, but move on if no rows returned.
	if (pg_num_rows($r2) == 0) {
		continue;
	}
	echo "\nProcessing file-level identity $identity\n";
	// Create a comma-separated list for the next query.
	$nlist = pg_fetch_all_columns($r2, 0);
	if ($debug) {
		echo "nlist is:\n";
		print_r($nlist);
	}
	$nodes = "";
	foreach($nlist as $n) {
		$nodes.= $n.",";
	}
	// Remove the trailing comma.
	$nodes = trim($nodes, ',');
	if ($debug) echo "nodes string is $nodes\n";
	// Find the number of backups associated with each. Typically the bogus
	// duplicates only have one backup. The outer join is required to get
	// those with no backups.
	$q = "SELECT node_no, COUNT(backup_no) FROM bp.nodes LEFT OUTER JOIN ";
	$q.= "bp.backups USING(node_no) WHERE node_no IN(".$nodes.") ";
	$q.= "GROUP BY node_no ORDER BY count";
	$r2 = pg_query($dbconn, $q);
	if ($r2 === false) {
		echo "ERROR: failed to query for backups of duplicate grandclients:\n";
		print pg_last_error($dbconn)."\n";
		bp_destroy_cookie();
		exit(1);
	}
	$nclients = pg_num_rows($r2);
	// Delete each duplicate client (except the one with the most backups).
	for ($j = 0; $j < $nclients; $j++) {
		$client_id = pg_fetch_result($r2, $j, "node_no");
		$nbkups = pg_fetch_result($r2, $j, "count");
		// Process all but the last one, unless it has no backups.
		if ($j == ($nclients - 1) && $nbkups > 0) {
			echo "Keep node_no $client_id $identity with $nbkups backup(s).\n";
			continue;
		}
		echo "Deleting node_no $client_id with $nbkups backup(s).\n";
		$ret = bp_delete_client($client_id);
		if ($ret === false) {
			 echo "bp_delete_client returned FALSE\n";
			  echo "error is: ".bp_error()."\n";
		}
	}
}

// Done with BPL calls.
bp_destroy_cookie();
echo "Grandclient processing complete.\n\n";

// Now find any duplicate application instances that might exist (app_id != 1)
$q = "SELECT identity FROM (SELECT COUNT(node_no), identity ";
$q.= "FROM bp.nodes JOIN bp.application_instances USING(node_no) ";
$q.= "WHERE (properties & 32)=32 AND app_id <> 1 GROUP BY identity) AS foo ";
$q.= "WHERE COUNT > 1";
$res = pg_query($dbconn, $q);
if ($res === false) {
	echo "ERROR: failed to query for duplicate application instances:\n";
	print pg_last_error($dbconn)."\n";
	exit(1);
}

$nrows = pg_num_rows($res);
printf("%d duplicated application instance identit%s found.\n", $nrows, 
		$nrows == 1 ? "y" : "ies");
for ($i = 0; $i < $nrows; $i++) {
	if (pg_field_is_null($res, $i, "identity")) {
		echo "Skipping NULL identity.\n";
		continue;
	}
	// Get the list of application instances with this identity.
	$identity = pg_fetch_result($res, $i, "identity");
	$q = "SELECT instance_id FROM bp.application_instances WHERE identity = '";
	$q.= $identity . "'";
	$r2 = pg_query($dbconn, $q);
	if ($r2 === false) {
		echo "ERROR: query failed for instance_ids of duplicate instances: \n";
		print pg_last_error($dbconn)."\n";
		exit(1);
	}
	// This shouldn't happen, but move on if no rows returned.
	if (pg_num_rows($r2) == 0) {
		continue;
	}
	echo "\nProcessing application identity $identity\n";
	// Create a comma-separated list for the next query.
	$ilist = pg_fetch_all_columns($r2, 0);
	if ($debug) {
		echo "ilist is:\n";
		print_r($ilist);
	}
	$inst = "";
	foreach($ilist as $a) {
		$inst.= $a.",";
	}
	// Remove the trailing comma
	$inst = trim($inst, ',');
	if ($debug) echo "instance string is $inst\n";
	// Find the number of backups associated with each.
	$q = "SELECT instance_id, COUNT(backup_no) FROM bp.application_instances ";
	$q.= "LEFT OUTER JOIN bp.backups USING(instance_id) WHERE instance_id IN(";
	$q.= $inst.") GROUP BY instance_id ORDER BY count";
	$r2 = pg_query($dbconn, $q);
	if ($r2 === false) {
		echo "ERROR: failed to query for backups of duplicate app instances:\n";
		print pg_last_error($dbconn)."\n";
		exit(1);
	}
	$ninst = pg_num_rows($r2);
	// Set the backups as logically deleted, then delete the instance record.
	for ($j = 0; $j < $ninst; $j++) {
		$inst_id = pg_fetch_result($r2, $j, "instance_id");
		$nbkups = pg_fetch_result($r2, $j, "count");
		// Process all but the last one, unless it has no backups.
		if ($j == ($ninst - 1) && $nbkups > 0) {
			echo "Keep instance_id $inst_id $identity with $nbkups backup(s).\n";
			continue;
		}
		echo "Logically delete $nbkups backup(s) for instance_id $inst_id\n";
		$q = "UPDATE bp.backups SET tape_no = 4294967295 WHERE instance_id = ";
		$q.= $inst_id;
		$rdel = pg_query($dbconn, $q);
		if ($rdel === false) {
			echo "ERROR: failed to logically delete backups for instance_id $inst_id:\n";
			print pg_last_error($dbconn)."\n";
			// what to do here, continue?
		}
		$q = "DELETE FROM bp.application_instances WHERE instance_id = ";
		$q.= $inst_id;
		$rdel = pg_query($dbconn, $q);
		if ($rdel === false) {
			echo "ERROR: failed to delete instance_id $inst_id:\n";
			print pg_last_error($dbconn)."\n";
			// what to do here, continue?
		}
	}
}
echo "Application instance processing complete.\n\n";

// Last check to make sure all duplicates are removed before adding the
// uniqueness constraint.
$q = "SELECT COUNT(node_no) FROM bp.nodes JOIN bp.application_instances ";
$q.= "USING(node_no) WHERE (properties & 32)=32 AND identity IS NOT NULL ";
$q.= "GROUP BY IDENTITY ORDER BY COUNT DESC LIMIT 1";
$res = pg_query($dbconn, $q);
if ($res === false) {
	echo "ERROR: query for last check of duplicates failed:\n";
	print pg_last_error($dbconn)."\n";
	exit(1);
}
if ((pg_num_rows($res) == 0) || (pg_fetch_result($res, 0, "count") <= 1)) {
	echo "No remaining duplicates exist.\n";
	$q = "ALTER TABLE bp.application_instances ADD CONSTRAINT ";
	$q.= "application_instances_identity_unique UNIQUE(identity)";
	$res = pg_query($dbconn, $q);
	if ($res === false) {
		echo "ERROR: failed to set uniqueness constraint on identity:\n";
		print pg_last_error($dbconn)."\n";
		exit(1);
	}
	else {
		echo "Succesfully added uniqueness constraint on identity.\n";
		exit (0);
	}
}
else {
	$n = pg_fetch_result($res, 0, "count");
	echo "Last check shows $n duplicate(s) still exist, not adding constraint.\n";
	exit (1);
}

?>
