#!/bin/sh
# Enabling debug on this script will return false negatives for winexe commands.
#
# Low-level support for Agent Push
# This script is designed to always return before the end of the script.  The
# last return is an indeterminate error and the script is in need of modification.
#
############################################################################
#Functions

function convertLog
{
	if [ -e $1 ]
	then
		tmpFile=/tmp/AgentPushReencode$$
		#iconv -f UTF16 -t US-ASCII $1 -o $tmpFile
		iconv -f $2 -t $3 $1 -o $tmpFile
		dos2unix $tmpFile
		mv -f $tmpFile $1
	fi
}

############################################################################
# main program starts here
#

winexe=/usr/bp/bin/winexe10-1

logFile=/usr/bp/logs.dir/cmc_AgentPush.log

touch $logFile

usage_str="Usage: `basename $0` <action> [args]

	action                arguments                                                                                  description
	------                ---------                                                                                  ---------------------------------------
	getLocalClientVersion win32|win64|winBM                                                                          get local client version
	winexeCmd msiExecUninstall username password domain clientIP/name dpuIP dpuName filename unininstallCmd          uninstall a msi package on a client
	winexeCmd msiExecReinstall username password domain clientIP/name dpuIP dpuName filename unininstallCmd corePath reinstall a msi package on a client
	winexeCmd msiExecInstall   username password domain clientIP/name dpuIP dpuName filename                         upgrade a msi package on a client
	winexeCmd prereqScript     username password domain clientIP/name dpuIP dpuName                                  check agent installation prerequisites
	
	argument           description
	--------           -----------
	filename(no ext) - MSI package to push without its file extension
	uninstallCmd     - this is the MSI command to uninstall the existing agent
	corePath         - existing location of PCBP, useful when non-default install location
	"

if [ $# -eq 0 ]
then
	echo "$usage_str" >&2
	exit 1
fi

case $1 in
	getLocalClientVersion)

		if [ $# -ne 2 ]
		then
			echo "getLocalClientVersion option requires client type argument." >&2
			exit 1
		fi

		case $2 in
			win32)
				filePath='/backups/AgentUpgrade/Agents/Windows/Unitrends_Agentx86.msi'
				agentName='Unitrends Agent'
			;;
			win64)
				filePath='/backups/AgentUpgrade/Agents/Windows/Unitrends_Agentx64.msi'
				agentName='Unitrends Agent'
			;;
			winBM)
				filePath='/backups/AgentUpgrade/Agents/Windows/Unitrends_BareMetal.msi'
				agentName='Bare Metal'
			;;
		esac
			
		if [ -e $filePath ]
		then
			versionNumber=$(strings -30 $filePath | grep -o -m 1 -e "$agentName [0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{8,\}" | awk -F '[. ]' '{print $3"."$4"."$5"-"$6}')
			if [ ${#versionNumber} -eq 0 ]
			then
				versionNumber=$(strings -30 $filePath | grep -o -m 1 -e "$agentName [0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{1,2\}\.[0-9]\{8,\}" | awk -F '[. ]' '{print $3"."$4"."$5}')
			fi
			if [ ${#versionNumber} -ne 0 ]
			then
				echo $versionNumber
				exit 0
			fi
		fi
		
		echo "Agent installer is not present on the targeted system." >&2
		exit 1
		;;

	winexeCmd)
		
			echo "$(date) PID: $$ Args: $1 $2 $3 [PasswordArg] ${@:5}" >> $logFile

			if [[ ( $2 == msiExecReinstall || $2 == msiExecInstall ) && ! -e "/backups/AgentUpgrade/Agents/Windows/$9.msi" ]]
			then
				echo "Agent installer $6 not present on system." >> $logFile
				echo "Agent installer $6 not present on system." >&2
				exit 1
			fi
			
			# First make sure Samba is running
			chkconfig --list smb | grep -q "on"
			if [ $? -ne 0 ]
			then
				echo "This command requires Samba to be enabled. (Settings->System, Updates, and Licensing->Support Toolbox->Samba On-Off)." >&2
				exit 1
			fi

			# Create temporary credential file
			credentialFile=/tmp/winexeTemp$$
			rm -f $credentialFile
			touch $credentialFile

			echo "username=$3" >> $credentialFile
			password=$(echo -n $4 | base64 -di)
			echo "password=$password" >> $credentialFile
			
			if [ "$5" != "DomainIsNULL" ]
			then
				echo "domain=$5" >> $credentialFile
			fi

			inputFile=/tmp/winexeStdIn$$.txt
			rm -f $inputFile
			if [[ ! -p $inputFile ]]; then
				mkfifo $inputFile
			fi

			outputFile=/tmp/winexeStdOutErr$$.txt

			rm -f $outputFile
			touch $outputFile

			/usr/bp/bin/winexeBackground.sh $$ &

			exec < $inputFile    # Will read from input file.

			exec 11>&1             #save stdout
			#exec > $outputFile   # Will write to output file.
														# Assumes output file writable.
			exec 12>&2             #save stderr
			exec > $outputFile 2>&1   #errors to output file

			# To log winexe, use log level -d 8. Anything above 0 will output false positive "ERROR:" strings
			# which are used in bpsql to determine whether the call failed.

			# this line reinstalls the winexesvc which should resolve the issue whereby the service is set to manual and a restart happens.
			$winexe -A $credentialFile --runas-file=$credentialFile --ostype=2 --reinstall //$6 "net view $8"

			netShowTestShareString=$(grep "The command completed successfully." $outputFile)
			netShowTestStringFound=$?

			if [ $netShowTestStringFound -eq 0 ]
			then
				echo "Name resolution test succeeded." >> $logFile
				dpu=$8
			else
				echo "Name resolution test failed, using IPv4 address." >> $logFile
				dpu=$7
			fi

			case $2 in
				msiExecUninstall)
					uninstallCmd=$(echo "${10}" | sed 's_/I_/X_')
					rm -f /backups/samba/$6_Uninstall_$9.log
					$winexe -A $credentialFile --runas-file=$credentialFile --ostype=2 --uninstall //$6 "cmd.exe /c start /wait $uninstallCmd /quiet /norestart /l* \\\\$dpu\\samba\\$6_Uninstall_$9.log"
					convertLog /backups/samba/$6_Uninstall_$9.log 'UTF16' 'US-ASCII'
				;;
				msiExecReinstall)
					uninstallCmd=$(echo "${10}" | sed 's_/I_/X_')
					rm -f /backups/samba/$6_*_$9.log
					$winexe -A $credentialFile --runas-file=$credentialFile --ostype=2 //$6 "cmd.exe /c start /wait $uninstallCmd /quiet /norestart /l* \\\\$dpu\\samba\\$6_Uninstall_$9.log"
					convertLog /backups/samba/$6_Uninstall_$9.log 'UTF16' 'US-ASCII'
					grep 'success or error status: 0.' /backups/samba/$6_Uninstall_$9.log
					successCode=$?
					grep 'Removal completed successfully' /backups/samba/$6_Uninstall_$9.log
					successMessage=$?
					if [[ $successCode == 0 || $successMessage == 0 ]]
					then
						$winexe -A $credentialFile --runas-file=$credentialFile --ostype=2 --uninstall //$6 "cmd.exe /c start /wait msiexec /quiet /norestart TARGETDIR=\"${11}\" MYAPPPATH=\"${11}\" /l* \\\\$dpu\\samba\\$6_Install_$9.log /I \\\\$dpu\\windows_agents\\$9.msi"
						convertLog /backups/samba/$6_Install_$9.log 'UTF16' 'US-ASCII'
					fi
				;;
				msiExecInstall)
					rm -f /backups/samba/$6_Install_$9.log
					$winexe -A $credentialFile --runas-file=$credentialFile --ostype=2 --uninstall //$6 "cmd.exe /c start /wait msiexec /quiet /norestart /l* \\\\$dpu\\samba\\$6_Install_$9.log /I \\\\$dpu\\windows_agents\\$9.msi"
					convertLog /backups/samba/$6_Install_$9.log 'UTF16' 'US-ASCII'
				;;
				prereqScript)
					rm -f /backups/samba/$6.xml
					$winexe -A $credentialFile --runas-file=$credentialFile --ostype=2 //$6 "cmd.exe /c start /b /wait \\\\$dpu\\agent_prereq\\mountAndRun.cmd $dpu $6"
					convertLog /backups/samba/$6.xml 'ISO_8859-1' 'UTF8'
				;;
				*)
					echo "Invalid operation: $1 $2" >&2
					exit 1
				;;
			esac

			exec 2>&12 12>&-       # Restore stderr
			exec 1>&11 11>&-       # Restore stout.
			
			# Delete temporary credential file
			rm -f $credentialFile $inputFile

			if [ "$2" == "prereqScript" ]
			then
				if [ ! -e /backups/samba/$6.xml ]
				then
					echo Prereq output file /backups/samba/$6.xml is missing. >> $logFile
					echo Prerequisite check output is missing. >&2
				else
					scriptOutput=$(head -1 /backups/samba/$6.xml)
					scriptOutputLength=$(expr length "$scriptOutput")
					echo "XML content: $scriptOutput" >> $logFile
					if [ $scriptOutputLength -gt 0 ]
					then
						rm -f "/backups/samba/$6.xml" $outputFile
						echo $scriptOutput
						exit 0
					fi
				fi
			fi
			
			if [[ "$2" == "msiExecInstall" || "$2" == "msiExecReinstall" ]]
			then
				if [ ! -e /backups/samba/$6_Install_$9.log ]
				then
					echo Install log file /backups/samba/$6_Install_$9.log is missing. >> $logFile
					echo Install log file /backups/samba/$6_Install_$9.log is missing. >&2
				else
					msiexecStatusString=$(grep 'Installation success or error status' /backups/samba/$6_Install_$9.log)
					statusStringFound=$?
					if [ $statusStringFound -ne 0 ]
					then
						msiexecStatusString=$(grep 'Installation completed successfully' /backups/samba/$6_Install_$9.log)
						installSuccess=$?
					fi
					
					echo Install status: $msiexecStatusString  >> $logFile
					
					# If success, return. If the message is empty continue on to find an error, else return with this error.
					successRegex='status: 0.'
					if [[ $installSuccess == 0 || $msiexecStatusString =~ $successRegex ]]
					then
						rm -f $outputFile /backups/samba/$6_*_$9.log
						echo $msiexecStatusString
						exit 0
					elif [[ ${#msiexecStatusString} -gt 0 && $statusStringFound -eq 0 ]]
					then
						rm -f $outputFile /backups/samba/$6_*_$9.log
						echo $msiexecStatusString | sed 's/[.]//g' | awk '{print "msi:install:"$NF}' >&2
						exit 1
					fi
					
					echo No success or error, dumping install log >> $logFile
					cat /backups/samba/$6_Install_$9.log >> $logFile
					rm -f /backups/samba/$6_Install_$9.log
					installFailed=1
				fi
			fi
			
			if [[ "$2" == "msiExecUninstall" || "$2" == "msiExecReinstall" ]]
			then
				if [ ! -e /backups/samba/$6_Uninstall_$9.log ]
				then
					echo Uninstall log file /backups/samba/$6_Uninstall_$9.log is missing. >> $logFile
					echo Uninstall log file /backups/samba/$6_Uninstall_$9.log is missing. >&2
				else
					msiexecStatusString=$(grep 'Removal success or error status' /backups/samba/$6_Uninstall_$9.log)
					statusStringFound=$?
					if [ $statusStringFound -ne 0 ]
					then
						msiexecStatusString=$(grep 'Removal completed successfully' /backups/samba/$6_Uninstall_$9.log)
						uninstallSuccess=$?
					fi
					
					echo Uninstall status: $msiexecStatusString >> $logFile
					
					# If success, return. If the message is empty continue on to find an error, else return with this error.
					successRegex='status: 0.'
					if [[ $uninstallSuccess == 0 || $msiexecStatusString =~ $successRegex ]]
					then
						rm -f $outputFile /backups/samba/$6_*_$9.log
						echo $msiexecStatusString
						
						# $installFailed will be set to 1 when we are reinstalling and the reinstall fails. Return error in that case. 
						if [ $installFailed -eq 1 ]
						then
							exit 1
						else
							exit 0
						fi
					elif [[ ${#msiexecStatusString} -gt 0 && $statusStringFound -eq 0 ]]
					then
						rm -f $outputFile /backups/samba/$6_*_$9.log
						echo $msiexecStatusString | sed 's/[.]//g' | awk '{print "msi:uninstall:"$NF}' >&2
						exit 1
					fi
					
					echo No success or error, dumping uninstall log >> $logFile
					cat /backups/samba/$6_Uninstall_$9.log >> $logFile
					rm -f /backups/samba/$6_Uninstall_$9.log
				fi
			fi

			winexeErrorString=$(grep "ERROR:" $outputFile)
			winexeErrorFound=$?
			
			if [ $winexeErrorFound -eq 0 ]
			then
				echo "Winexe error found: $winexeErrorString" >> $logFile
				rm -f $outputFile
				echo $winexeErrorString >&2
				exit 1
			else
				echo "Either execution succeeded and there was an error writing or reading the log file, or this is an unknown error. Script output: " >> $logFile
				sed -i "s/$password/PasswordRemoved/g" $outputFile
				cat $outputFile >> $logFile
				rm -f $outputFile
				echo Unable to upgrade agent. See cmc_AgentPush.log for logging. Reference Unitrends KB 693 for more information. >&2
				exit 1
			fi
		;;

		*)
			echo "Invalid operation: $1" >&2
			exit 1
		;;
esac
