#!/bin/sh
# Change log entries
# Bug# Date    Who Description
# ---- ------- --- ------------------------------------------------------
# 7515 04Jun09 RJM Initial implementation
#
# Project: development
# File:    enable core dump in field
# Type:    POSIX.2 /bin/sh Shell Command Language
#          FHS-2.2
# Dev:     none yet
# Usage:   insert call into bp/rpm/SPECS/postinstall
#          $0 >> $LOG 2>&1
# Biblio:  POSIX.2 IEEE 1003.2
#          FHS-2.2 http://www.pathname.com/fhs/
#          HOWTO enable core-dumps in fedora/CentOS
#            http://en.linuxreviews.org/HOWTO_enable_core-dumps 
#          Documentation for /roc/sys/kernel/*
#            core_pattern:
#              /usr/share/doc/kernel-doc-2.6.18/Documentation/sysctl/kernel.txt
#                package kernel-doc:




set -o posix



#
# Constants
#

# Parameter Defaults
            HELP=false
         VERBOSE=false
           PURGE=false

   CAPACITYLIMIT="14:1000000:85"
      COREPREFIX="/backups/dumps/core"
           LIMIT="unlimited"
         DEFAULT="0"

            DATE="$(date +%Y%m%d)"

# Exit Codes (Berkeley standard sysexits)
          EX_OK=0               # everything worked
       EX_USAGE=64              # incorrect arguments
     EX_DATAERR=65              # file is corrupt
                                # (e.g. certificate or private key)
     EX_NOINPUT=66              # file non-existent or unreadable
 EX_UNAVAILABLE=69              # unable to edit config file
    EX_SOFTWARE=70              # assert failed (internal error)
   EX_CANTCREAT=73              # unable to mv old file
       EX_IOERR=74              # unable to move new file



#
# Support
#

# Write a string to stderr
# @param $* message for stderr
echoe() { echo "$*"; } 1>&2

# Alter a configuration file
#   If alteration is feckless, leave original unaltered and remove new files
#   If alteration succeeds, move original and replace with new file
#   If alteration fails, leave original and create new file
# 
# @param $1 file to alter (e.g. /etc/profile)
# @param $2 perl s::: expression for alteration
#           (e.g. 's:^(\s*ulimit\s+-S\s+-c\s+)([^\s]+):\1unlimited:;')
# @param $3 empty or grep -E pattern which if not found in pre-existing file
#                                    signals failure (i.e.dont change)
#           (e.g. '^ulimit -S -c [1-9][0-9]*'
# @param $4 empty or grep -E pattern which if found in pre-existing file
#                                    signals failure (i.e. dont change)
#           (e.g. '^ *DAEMON_COREFILE_LIMIT *=')
# @param $5 empty or grep -E pattern which if found indicates
#                                    alteration success
#           (e.g. '^ulimit -S -c [1-9][0-9]*'
# @param $6 empty or line(s) to append if alteration does not succeed
# @param $7 empty (defaults to *.Unitrends.yyyymmdd) or new filename on failure
# @param $8 empty or original destination on success
# @retval $EX_OK change succeeded
# @retval $EX_CANTCREAT feckess change not made
# @retval $EX_DATAERR verification failed, change not made
# @retval $EX_SOFTWARE case not handled
# @retval $EX_UNAVAILABLE unable to perform alteration, change not made
# @retval $EX_IOERR unable to write or move file
AlterConfFile() {
  _FILE="$1"
  _SCOMMAND="$2"
   _FAILPAT="$3"
  _FAILNPAT="$4"
   _SUCCPAT="$5"
    _APPEND="$6"
   _NEWFILE="$7"
   _OLDFILE="$8"
  error=$EX_SOFTWARE

  # Set default values
  if test -z "$_NEWFILE"
  then
    _NEWFILE="$_FILE.Unitrends.""$(date +%Y%m%d)"
  fi

  # Apply alteration but dont alter orignal
  if ! perl -pe "$_SCOMMAND" "$_FILE" > "$_NEWFILE"
  then
    echoe "Error: unable to edit or write $_NEWFILE ."
    echoe "Warning: $_FILE not changed"
    error=$EX_UNAVAILABLE
  elif test -n "$_APPEND" && test -n "$_SUCCPAT" && ! grep -E "$_SUCCPAT" "$_NEWFILE" >/dev/null && ! echo "$_APPEND" >> "$_NEWFILE"
  then
    # Append if given and needed
    echoe "Error: unable to append to $_NEWFILE ."
    echoe "Warning: $_FILE not changed"
    error=$EX_IOERR

  # Verify alteration before altering original
  elif test -n "$_SUCCPAT" && grep -E "$_SUCCPAT" "$_NEWFILE" >/dev/null && cmp "$_FILE" "$_NEWFILE" >/dev/null && rm "$_NEWFILE"
  then
    # Accept verified alteration that is feckless as success
    error=$EX_CANTCREAT
    true
  elif test -n "$_FAILPAT" && ! grep -E "$_FAILPAT" "$_FILE" >/dev/null
  then
    # Reject alteration if expectations unmet
    echoe "Warning: $_FILE failed to match ($_FAILPAT)"
    echoe "Warning: $_FILE not changed"
    echoe "info: $_NEWFILE created"
    error=$EX_DATAERR
  elif test -n "$_FAILNPAT" && grep -E "$_FAILNPAT" "$_FILE" >/dev/null
  then
    # Reject alteration if expectations unmet
    echoe "Warning: $_FILE matched ($_FAILNPAT)"
    echoe "Warning: $_FILE not changed"
    echoe "info: $_NEWFILE created"
    error=$EX_DATAERR
  elif test -n "$_SUCCPAT" && ! grep -E "$_SUCCPAT" "$_NEWFILE" >/dev/null
  then
    # Failed to verify success
    echoe "Error: unable to edit $_NEWFILE ."
    echoe "Warning: $_FILE not changed"
    error=$EX_DATAERR

  # Leave alteration in place as well and rename different original
  elif cmp "$_FILE" "$_NEWFILE" >/dev/null && rm "$_NEWFILE"
  then
    # If change is feckless, dont leave duplicate file around
    error=$EX_CANTCREAT
    true
  elif test -n "$_OLDFILE" && ! mv "$_FILE" "$_OLDFILE"
    # At this point, either _SUCCPAT is empty, and cmp failed
    # or _SUCCPAT was nonempty and satisfied, and cmp failed
  then
    echoe "Error: unable to rename $_FILE -> $_OLDFILE ."
    echoe "Warning: $_FILE not changed"
    echoe "info: $_NEWFILE created"
    error=$EX_IOERR
  elif ! mv "$_NEWFILE" "$_FILE"
  then
    echoe "Error: unable to replace $_FILE ."
    echoe "Warning: $_FILE not changed"
    echoe "info: $_NEWFILE created"
    error=$EX_IOERR
  else
    error=$EX_OK
  fi

  return $error
}

# Make core path directory if not already present
MakeCorePath() {
  _PREFIX="$1"
  if ! test -n "$_PREFIX"
  then
    true
  elif ! mkdir -m 1733 -p $(dirname "$_PREFIX")
  then
    echoe "Error: unable to create directory, "$(dirname "$_PREFIX")
    error=$EX_IOERR
  else
    error=$EX_OK
  fi

  return $error
}

# Remove cores that are too old
# @param $1 max age/24h : max dir size / 1Kio ; max fs used / %
# @param $2 pattern to match for purge candidates
PurgeCores() {
  _LIMITS="$1"
  _PREFIX="$2"
  _DIR="$(dirname "$_PREFIX")"
  _AGEDAYS="$(echo "$_LIMITS" | cut -d: -f1)" # Extract max allowed days
  _DIRMAX="$(echo "$_LIMITS" | cut -d: -f2)" # Extract max directory Kio
  _FSMAX="$(echo "$_LIMITS" | cut -d: -f3)" # Extract max percentage used
  if test -d "$_DIR" -a -r "$_DIR"
  then
    # One-by-one if too old, purge core files
    find "$_PREFIX".* -mtime +$_AGEDAYS 2>/dev/null | while read f
    do
      rm "$f"
    done
    # One-by-one randomly purge files until capacity limits are satisfied
    find "$_PREFIX".* -type f -print 2>/dev/null | while read p
    do
      if test $(du -sk "$_DIR" | # Query disk usage
	      cut -f1		# Extract only the Kio
	  ) -gt "$_DIRMAX"	# Compare
      then
	# directory is too large
	rm "$p"
      elif test $(df -P "$_DIR" | # Query filesystem percentage used
	      tail -1 |		# Ignore the headers
	      perl -pe 's: +:\t:g;' | # Change delimiter spaces to tab
	      cut -f5 |		# Extract Used field
	      perl -pe 's:[^0-9]::g;') \
	  -gt "$_FSMAX"		# Compare
      then
	# filesystem is too full
	rm "$p"
      else
        break
      fi
    done
  fi
}

# Compress cores to save space
CompressCores() {
  _PREFIX="$1"
  if test -d $(dirname "$_PREFIX") -a -r $(dirname "$_PREFIX")
  then
    # One by one, if not already compressed, compress cores
    find "$_PREFIX".* -type f 2>/dev/null | while read f
    do
      if file "$f" | 		# Identify contents heuristically
	  grep compressed 2>/dev/null 1>/dev/null # compressed?
      then
        true
      else
        nice -15 gzip "$f"
      fi
    done
  fi
}

# Enable daemon core dumps
#
# See http://en.linuxreviews.org/HOWTO_enable_core-dumps 
# Fedora/CentOS daemon core dumps are enabled by setting
#   DAEMON_COREFILE_LIMIT in /etc/sysconfig/<daemon>
#
# @param $1 new core size limit
EnableDaemonCoreDump() {
  _LIMIT="$1"

  # bpserverd is called from xinetd
  # RRC is supported by httpd
  for f in "/etc/sysconfig/xinetd" "/etc/sysconfig/httpd"
  do
    # CentOS-5 magic
            _TAG="DAEMON_COREFILE_LIMIT" # used by /etc/init.d/<daemon>
	    _MSK="^ *$_TAG="	# already set if found by grep -E
	                        # Alteration performed by perl
            _CHG='s:^(\s*'"$_TAG"'=)([^\s]*):\1'"$_LIMIT"':;'
            _APP="$(cat <<EOF	# what to append (if not already set)

# Unitrends Rapid Recovery Console
# Enable core dump
$_TAG=$_LIMIT
EOF
)"

    # For each file (f), apply (CHG), (no prior requirement ()),
    # if not alread set (MSK), and if not successful (MSK), append (APP),
    # to write (*.Unitrends.date) on failure,
    # or save old file as (*.date) on success.
    AlterConfFile "$f" "$_CHG" '' "$_MSK" "$_MSK" "$_APP" \
                  "$f.Unitrends.$DATE" ''
  done
}

# Enable kernel core dumps
#
# Fedora/CentOS kernel core dumps are enabled by setting
#   /etc/sysctl.conf
#
# @param $1 pattern for core file to dump
EnableKernelCoreDump() {
  _PREFIX="$1"

  # cores are directed to one directory by kernel configuration
  f="/etc/sysctl.conf"

  # linux-2.4 magic (See Documentation for /roc/sys/kernel/*)
            _PRP="%p.%e.%t"	# process id, executable, and time
                                #   of core dump
  # See AlterConfFile description above for further details
            _TAG="kernel.core_pattern" # used by sysctl(8)
            _MSK="^ *$_TAG *="  # already set if found by grep -E
                                # Alteration performed by perl
            _CHG='s:^ *'"$_TAG"' *=.*$:'"$_TAG"'='"$_PREFIX.$_PRP"':;'
            _SUC="^ *$_TAG *= *$_PREFIX.$_PRP" # success if found by grep -E
            _APP="$(cat <<EOF   # what to append (if not already set)
# Unitrends Rapid Recovery Console
$_TAG=$_PREFIX.$_PRP
EOF
)"

  # For each file (f), apply (CHG), if not already set (MSK),
  # nothing else required (), and if not successful (MSK), append (APP),
  # to write (*.Unitrends.date) on failure,
  # or save old file as (*.date) on success.
  AlterConfFile "$f" "$_CHG" '' "$_MSK" "$_SUC" "$_APP" \
                "$f.Unitrends.$DATE" ''

  # avoid net.bridge error messages (UNIBP-1890)
  lsmod |grep -q bridge
  if [ $? -ne 0 ]; then
     modprobe bridge
  fi
  # Read changed config file into running kernel.
  test $EX_OK -eq $? && /sbin/sysctl -p "$f" >/dev/null
}


# Parse command line
error=$EX_OK
while getopts ":hvca:l:p:" opt
do
  case "$opt" in
  h)                         HELP=true;;
  v)                      VERBOSE=true;;
  c)                        PURGE=true;;
  a)                CAPACITYLIMIT="$OPTARG";;
  l)                        LIMIT="$OPTARG";;
  p)                       PREFIX="$OPTARG";;
  *) echoe "Error: unknown option: '$opt'"
     error=$EX_USAGE;;
  esac
done



#
# Validate parameters
#

# None yet



#
# Report usage if necessary
#
if $HELP || test $EX_OK -ne $error
then
  cat <<EOF 1>&2
usage: $0 [-v] -h
       $0 [-v] -c [-a days:Kio:pct] [-p pathname]
       $0 [-v] [-l {unlimited|sizekio}]
               [-m {unlimited|sizekio|regex}]
               [-p pathname]
  -a purge after days*24       [$CAPACITYLIMIT]
     until dir size < Kio
     and until fs pct < pct
  -c purge files in dump       [$PURGE]
  -h print this message        [$HELP]
  -l new user core limit       [$LIMIT]
  -m required current limit    [$DEFAULT]
  -p core path prefix          [$COREPREFIX]
  -v report in greater detail  [$VERBOSE]
EOF
  if $VERBOSE
  then
  cat <<EOF 1>&2

Change the user core limit configuration to enable core dumps
if and only if the current setting is the package pam default.

Changes do not affect currently running executables.
Restart xinetd for service cores
Relogin for cli cores.
Reboot for kernel cores.

Redirect cores to core path prefix takes place immediately.

Return Codes:
  $EX_OK  - everything worked
  $EX_USAGE - incorrect arguments
  $EX_DATAERR - current setting is not package pam default
  $EX_NOINPUT - file non-existent or unreadable
  $EX_UNAVAILABLE - file alteration failed
  $EX_IOERR - unable to update file
EOF
  fi
fi


#
# Perform
#

if test $EX_OK -ne $error
then
  true            # Abort on error
elif $HELP
then
  true            # Already reported
elif $PURGE
then
  # Purge cores older than specified days
  PurgeCores "$CAPACITYLIMIT" "$COREPREFIX"
  # Compress cores not alrady compressed
  CompressCores "$COREPREFIX"
else
  # Configure daemons (e.g. bpserverd via /etc/init.d/xinetd) to dump cores
  test -n "$LIMIT" && EnableDaemonCoreDump "$LIMIT"

  # Create the directory to receive core dumps
  test -n "$COREPREFIX" && MakeCorePath "$COREPREFIX"

  # Configure Unitrends modified kernel to dump cores
  test -n "$COREPREFIX" && EnableKernelCoreDump "$COREPREFIX"
fi



#
# Cleanup
#

# nothing yet


exit $error
