<?
// If an error occurs during the transaction, this function is called to
// do a rollback, close the connection, and exit with error code 1.
function cleanup_and_exit($conn) {
	pg_query($conn, "ROLLBACK;");
	pg_close($conn);
	exit(1);
}

// Returns the sql necessary to set the workspace table's sequences to
// their proper values (max(PK)) for the specified schema
function get_sequence_sql($schema_name) {
$SQL = <<<EOSQL
	SELECT pg_catalog.setval('content_id_seq', (SELECT MAX(contentid) FROM "{$schema_name}".content), true);
	SELECT pg_catalog.setval('databank_id_seq', (SELECT MAX(databankid) FROM "{$schema_name}".databank), true);
	SELECT pg_catalog.setval('eventgroup_id_seq', (SELECT MAX(eventgroupid) FROM "{$schema_name}".eventgroup), true);
	SELECT pg_catalog.setval('eventlog_id_seq', (SELECT MAX(eventid) FROM "{$schema_name}".eventlog), true);
	SELECT pg_catalog.setval('folder_id_seq', (SELECT MAX(folderid) FROM "{$schema_name}".folder), true);
	SELECT pg_catalog.setval('folder_tree_id_seq', (SELECT MAX(foldertreeid) FROM "{$schema_name}".foldertree), true);
	SELECT pg_catalog.setval('item_id_seq', (SELECT MAX(itemid) FROM "{$schema_name}".item), true);

EOSQL;
	return $SQL;
}

function usage($conn) {
	echo "Usage: unitrends_pgdump.php <mode> <workspace_name> [dpuid]\n";
	echo "where mode is one of:  vaulting dpurestore archive\n\n";
	echo "Here is a list of current workspace names to choose from:\n";
	$result = pg_query($conn, "select workspacename from bp.workspace order by workspacename;");
	if ($result) {
   		for ($row = 0; $row < pg_numrows($result); $row++) {
       		  $fullname = pg_result($result, $row, 'workspacename') . " ";
        	  echo ($row + 1) . ": " . $fullname . "\n";
   		}
	} else {
   		echo "Query failed with the following error:\n";
   		echo pg_errormessage($conn);
	} 
} 

// Main program starts here
// Connect to the Postgres Database
$conn = pg_Connect("port=5432 dbname=bpdb user=postgres");
if (!$conn) {
  	echo "A database connection error occurred.\n"; 
	exit(1);
}

if ($argc < 3) {
	usage($conn);
	exit (1);
}

// "mode" indicates the context for this dump:
// vaulting - dump a DPU workspace to transfer to the vault in preparation for mirroring
// dpurestore - dump a vaulted workspace to restore on a new replacement DPU
// archive - produce a dump for archiving purposes
$mode = $argv[1];
if ($mode == "vaulting" or $mode == "dpurestore" or $mode == "archive") {
	echo "Dumping schema in $mode mode.\n";
} else {
	echo "Invalid mode: $mode.  Mode must be one of:  vaulting dpurestore archive\n";
	usage($conn);
	exit (1);
}

$workspace_to_dump = $argv[2];

$query = "select workspacename, dpuid from bp.workspace where workspacename = '$workspace_to_dump'";
if (isset($argv[3]) and ($argv[3] != '-')) {
	$query .= " and dpuid = '{$argv[3]}'";
	$dpustatusstring = " on DPU $argv[3]";
} else {
	$dpustatusstring = '';
}
$result = pg_query($conn, $query);
$nrows = pg_numrows($result);
if (!$nrows) {
	echo "$workspace_to_dump not found{$dpustatusstring}\n";
	usage($conn);
	exit(1);
} 

// Require a dpuid arg to be specified if needed.
if ($nrows > 1) {
	echo "There are $nrows workspaces named $workspace_to_dump.\n";
	echo "Please specify a DPU id:  unitrends_pgdump.sh <workspace_name> <dpuid>\n";
	while ($row = pg_fetch_assoc($result)) {
		echo "  " . $row['workspacename'] . " has DPU id " . $row['dpuid'] ."\n";
	}
	exit(1);
}


$dpuid = pg_result($result, 0 , 'dpuid');

$result = pg_query($conn, "select bpexch.getworkspaceschema('" . $workspace_to_dump . "','" . $dpuid . "')  as full_name ;");
$fullworkspace_name = pg_result($result, 0 , 'full_name');

// Find out if vaulting is currently enabled for this workspace.  This variable captures
// the state before the dump is performed.  Depending on the mode (vaulting, dpurestore,
// or archive), the initial state determines some of the actions that follow:
//
// When creating the initial dump to set up vaulting (mode=vaulting), if vaulting is
// initially OFF, after the dump is completed, it should be turned ON and the triggers
// should be enabled on the workspace tables.  If it is initially ON, it should be
// temporarily turned OFF during the transaction, and a statement should be appended to
// the dump to disable the triggers when the workspace is transferred to the vault.
// 
// For archive dumps (mode=archive), if vaulting is OFF for a workspace, it should be
// left OFF.  If it is ON, a statement should be appended to the dump to disable triggers
// when the workspace is restored.
//
// For DPU restore (mode=dpurestore), this setting will be OFF.

$query = "SELECT isvaulted::int FROM bp.workspace WHERE workspacename = '$workspace_to_dump' and dpuid = '$dpuid';";
$result = pg_query($conn, $query);
$workspace_vaulting_enabled = pg_result($result, 'isvaulted');
printf ("Workspace vaulting is currently %s.\n", $workspace_vaulting_enabled? "enabled":"disabled");

if (isset($argv[3]) and ($argv[3] != '-')) {
	$output_file_name = $workspace_to_dump . "_" . $dpuid . ".sql";
}
else {
	$output_file_name = $workspace_to_dump . ".sql";
}

// Begin the Transaction
$result = pg_query($conn, "begin work;");
if (!$result) {
	echo "An error occurred beginning the transaction.\n"; 
	exit(1);
}

// If this dump is being created to get the DPU and vault synchronized so that vaulting
// can begin, we don't want any transfers to occur until the dump has been replayed on the
// vault.  Turn off the global vault transfer setting in bp.config.  This will be turned 
// back on when unitrends_enable_vault_transfer.sh is called after the workspace has been
// created and populated on the vault.
if ($mode == "vaulting") {
	echo "Updating CONFIG table to disable vault transfer...\n";
	$result = pg_query($conn, "UPDATE BP.CONFIG SET value = 'no' where key = 'vault_transfer_enabled';");
	if (!$result) {
		echo "An error occurred setting vault_transfer_enabled to no.\n"; 
		cleanup_and_exit();
	}
	else {
		echo "CONFIG table updated. Vault transfer has been disabled.\n\n";
	}
}

echo "Locking tables...\n";
// Set search path
echo "Setting search path...\n";
$result = pg_query($conn, "set search_path to \"$fullworkspace_name\";");
if (!$result) {
	echo "An error occurred setting search path to $fullworkspace_name.\n";
	cleanup_and_exit($conn);
}

$result = pg_query($conn, "select bpexch.lockvaultedtables();");
if (!$result) {
	echo "An error occurred locking the $workspace_to_dump workspace{$dpustatusstring}.\n"; 
	cleanup_and_exit($conn);
}
echo "Tables locked.\n\n";

// I think this is really useless since we are in a transaction
if ($workspace_vaulting_enabled == TRUE) {
	echo "Turning off workspace vaulting...\n";
	pg_query($conn, "UPDATE bp.workspace SET isvaulted = false where workspacename = '$workspace_to_dump' and dpuid = '$dpuid';");
	if (!$result) {
		echo "An error occurred while turning off vaulting for $workspace_to_dump workspace{$dpustatusstring}.\n";
		cleanup_and_exit($conn);
	}
	echo "Workspace vaulting is off.\n\n";
}

// Dump database
$low_workspace = strtolower($workspace_to_dump);
$low_dpuid = strtolower($dpuid);

// Create, use and then destroy a function to drop the schema if it exists.
// If the schema does not exist and we simply use a 'drop schema' command to remove it,
// it will generate an error and cause the load of the dump to fail.
$function_stmts = "CREATE OR REPLACE FUNCTION dropMySchema(myschema varchar) RETURNS void AS $$ 
BEGIN
IF (select count(*) from pg_catalog.pg_namespace where nspname = myschema )>0 THEN
	RAISE NOTICE 'Function dropMySchema is dropping the % schema', myschema;
	EXECUTE 'DROP SCHEMA ' || quote_ident(myschema) || ' CASCADE';
END IF;
END;
$$ LANGUAGE 'plpgsql' VOLATILE;

SELECT dropMySchema('$fullworkspace_name');

DROP FUNCTION dropMySchema(varchar);\n\n";
// End of function-related statements

$delete_statement = "DELETE FROM bp.workspace WHERE workspacename = '$low_workspace' AND dpuid = '$low_dpuid' AND workspacetypeid = 1;\n\n";
$insert_statement = "INSERT INTO bp.workspace(workspacename,dpuid,workspacetypeid,description,creationdate,isvaulted) VALUES ('$low_workspace','$low_dpuid',1,'$workspace_to_dump',now(),false);\n\n";

$big_str = sprintf("BEGIN;\n\n%s%s%s", $function_stmts, $delete_statement,$insert_statement);

$fp = fopen($output_file_name, "w", 0); 
if ($fp) {
	if ( fputs($fp, $big_str) === FALSE) {
		echo "An error occurred writing to $output_file_name.\n";
		cleanup_and_exit($conn);
	}
	fclose($fp); 
}
else {
	echo "An error occurred opening $output_file_name for writing.\n";
	cleanup_and_exit($conn);
}


echo "Dumping {$workspace_to_dump}{$dpustatusstring} from bpdb database... \n";
exec("pg_dump -O -s -U postgres -h localhost -E WIN1252 -n $fullworkspace_name bpdb >> " . $output_file_name,$array_return,$exec_result);
if ($exec_result != "0") {
	echo "A pg_dump error occurred during schema dump phase.\n"; 
	print "ERROR: $array_return[0]\n";
	cleanup_and_exit($conn);
}
exec("pg_dump -O -a --disable-triggers -U postgres -h localhost -E WIN1252 -n $fullworkspace_name bpdb >> " . $output_file_name,$array_return,$exec_result);
if ($exec_result != "0") {
	echo "A pg_dump error occurred during data dump phase.\n"; 
	print "ERROR: $array_return[0]\n";
	cleanup_and_exit($conn);
}

// Now we need to append some stuff at the end.
$fp = fopen($output_file_name, "a", 0);
if ($fp) {
	// The --disable-triggers option temporarily prevents the triggers from firing while
	// the dump is being restored.  We want to permanently drop the triggers on the vault
	// copy of the workspace so that the dbmirror tables don't get populated every time 
	// records are mirrored over from the DPU.  For archiving, we similarly do not want
	// a workspace to have triggers enabled after restore or the dbmirror tables will
	// get continually filled and never flushed.
	// (In the dump, search_path is already set to (the workspace, pg_catalog).)
	if ($workspace_vaulting_enabled == TRUE) {
		$drop_trigger_statement = "SELECT bpexch.setdbmirrortriggers(false);\n\n";
		if ( fputs($fp, $drop_trigger_statement) === FALSE) {
			echo "An error occurred appending to $output_file_name.\n";
			cleanup_and_exit($conn);
		}
	}

	// Optionally add SQL to update the sequences because dbmirror'ing doesn't do that
	if ($mode == "dpurestore" or $mode == "archive") {
		$sequence_sql = get_sequence_sql($fullworkspace_name);
		if ( fputs($fp, $sequence_sql) === FALSE) {
			echo "An error occurred appending to $output_file_name.\n";
			cleanup_and_exit($conn);
		}
	}

	// Add the COMMIT to finish the transaction.
	if ( fputs($fp, "COMMIT;\n\n") === FALSE) {
		echo "An error occurred appending to $output_file_name.\n";
		cleanup_and_exit($conn);
	}
	fclose($fp);
}
else {
	echo "An error occurred opening $output_file_name for appending.\n";
	cleanup_and_exit($conn);
}

$path = rtrim(shell_exec('pwd'));
echo "Dump complete\n\n";
echo "**********************************************************\n";
echo "---> File [" . $path . "/" . $output_file_name . "] has been created. <--\n";
echo "**********************************************************\n\n";

if ($mode != "archive") {
	echo "Cleaning out pending dbmirror records...\n";
	echo "Re-setting search path...\n";
	$result = pg_query($conn, "set search_path to public;");
	if (!$result) {
		echo "An error occurred setting search path to public.\n"; 
		cleanup_and_exit($conn);
	}

	$result = pg_query($conn, "DELETE FROM dbmirror_Pending where tablename LIKE '%$fullworkspace_name%'");
	if (!$result) {
		echo "An error occurred deleting the $workspace_to_dump workspace records from dbmirror_Pending.\n"; 
		cleanup_and_exit($conn);
	}
	echo "Cleaning complete.\n\n";
}

if ($mode == "vaulting" or ($mode == "archive" and $workspace_vaulting_enabled == TRUE)) {
	echo "Turning on workspace vaulting...\n";
	pg_query($conn, "UPDATE bp.workspace SET isvaulted = true where workspacename = '$workspace_to_dump' and dpuid = '$dpuid';");
	if (!$result) {
		echo "An error occurred while turning on vaulting for $workspace_to_dump workspace{$dpustatusstring}.\n";
		cleanup_and_exit($conn);
	}
	echo "Workspace vaulting is on.\n\n";
}

// If the initial state of the workspace had vaulting OFF, add triggers to the
// tables so that they can be mirrored over to the vault.
if ($mode == "vaulting" and $workspace_vaulting_enabled == FALSE ) {
	echo "Creating triggers...\n";
	echo "Setting search path...\n";
	$result = pg_query($conn, "set search_path to \"$fullworkspace_name\";");
	if (!$result) {
		echo "An error occurred setting search path to $fullworkspace_name.\n";
		cleanup_and_exit($conn);
	}
	$result = pg_query($conn, "SELECT bpexch.setdbmirrortriggers(true);");
	if (!$result) {
		echo "An error occurred creating triggers on the $fullworkspace_name tables.\n";
		cleanup_and_exit($conn);
	}
	else {
		echo "Triggers created.\n\n";
	}
}

// End the transaction
pg_query($conn, "commit work;");

// Close the connection
pg_Close($conn);
exit(0);

?>
